# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._compat import cached_property
from ._version import __version__
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import SpottedError, APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

if TYPE_CHECKING:
    from .resources import (
        me,
        shows,
        users,
        albums,
        browse,
        search,
        tracks,
        artists,
        markets,
        chapters,
        episodes,
        playlists,
        audiobooks,
        audio_analysis,
        audio_features,
        recommendations,
    )
    from .resources.me.me import MeResource, AsyncMeResource
    from .resources.shows import ShowsResource, AsyncShowsResource
    from .resources.albums import AlbumsResource, AsyncAlbumsResource
    from .resources.search import SearchResource, AsyncSearchResource
    from .resources.tracks import TracksResource, AsyncTracksResource
    from .resources.artists import ArtistsResource, AsyncArtistsResource
    from .resources.markets import MarketsResource, AsyncMarketsResource
    from .resources.chapters import ChaptersResource, AsyncChaptersResource
    from .resources.episodes import EpisodesResource, AsyncEpisodesResource
    from .resources.audiobooks import AudiobooksResource, AsyncAudiobooksResource
    from .resources.users.users import UsersResource, AsyncUsersResource
    from .resources.browse.browse import BrowseResource, AsyncBrowseResource
    from .resources.audio_analysis import AudioAnalysisResource, AsyncAudioAnalysisResource
    from .resources.audio_features import AudioFeaturesResource, AsyncAudioFeaturesResource
    from .resources.recommendations import RecommendationsResource, AsyncRecommendationsResource
    from .resources.playlists.playlists import PlaylistsResource, AsyncPlaylistsResource

__all__ = ["Timeout", "Transport", "ProxiesTypes", "RequestOptions", "Spotted", "AsyncSpotted", "Client", "AsyncClient"]


class Spotted(SyncAPIClient):
    # client options
    access_token: str

    def __init__(
        self,
        *,
        access_token: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Spotted client instance.

        This automatically infers the `access_token` argument from the `SPOTIFY_ACCESS_TOKEN` environment variable if it is not provided.
        """
        if access_token is None:
            access_token = os.environ.get("SPOTIFY_ACCESS_TOKEN")
        if access_token is None:
            raise SpottedError(
                "The access_token client option must be set either by passing access_token to the client or by setting the SPOTIFY_ACCESS_TOKEN environment variable"
            )
        self.access_token = access_token

        if base_url is None:
            base_url = os.environ.get("SPOTTED_BASE_URL")
        if base_url is None:
            base_url = f"https://api.spotify.com/v1"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def albums(self) -> AlbumsResource:
        from .resources.albums import AlbumsResource

        return AlbumsResource(self)

    @cached_property
    def artists(self) -> ArtistsResource:
        from .resources.artists import ArtistsResource

        return ArtistsResource(self)

    @cached_property
    def shows(self) -> ShowsResource:
        from .resources.shows import ShowsResource

        return ShowsResource(self)

    @cached_property
    def episodes(self) -> EpisodesResource:
        from .resources.episodes import EpisodesResource

        return EpisodesResource(self)

    @cached_property
    def audiobooks(self) -> AudiobooksResource:
        from .resources.audiobooks import AudiobooksResource

        return AudiobooksResource(self)

    @cached_property
    def me(self) -> MeResource:
        from .resources.me import MeResource

        return MeResource(self)

    @cached_property
    def chapters(self) -> ChaptersResource:
        from .resources.chapters import ChaptersResource

        return ChaptersResource(self)

    @cached_property
    def tracks(self) -> TracksResource:
        from .resources.tracks import TracksResource

        return TracksResource(self)

    @cached_property
    def search(self) -> SearchResource:
        from .resources.search import SearchResource

        return SearchResource(self)

    @cached_property
    def playlists(self) -> PlaylistsResource:
        from .resources.playlists import PlaylistsResource

        return PlaylistsResource(self)

    @cached_property
    def users(self) -> UsersResource:
        from .resources.users import UsersResource

        return UsersResource(self)

    @cached_property
    def browse(self) -> BrowseResource:
        from .resources.browse import BrowseResource

        return BrowseResource(self)

    @cached_property
    def audio_features(self) -> AudioFeaturesResource:
        from .resources.audio_features import AudioFeaturesResource

        return AudioFeaturesResource(self)

    @cached_property
    def audio_analysis(self) -> AudioAnalysisResource:
        from .resources.audio_analysis import AudioAnalysisResource

        return AudioAnalysisResource(self)

    @cached_property
    def recommendations(self) -> RecommendationsResource:
        from .resources.recommendations import RecommendationsResource

        return RecommendationsResource(self)

    @cached_property
    def markets(self) -> MarketsResource:
        from .resources.markets import MarketsResource

        return MarketsResource(self)

    @cached_property
    def with_raw_response(self) -> SpottedWithRawResponse:
        return SpottedWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SpottedWithStreamedResponse:
        return SpottedWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        access_token = self.access_token
        return {"Authorization": f"Bearer {access_token}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        access_token: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            access_token=access_token or self.access_token,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncSpotted(AsyncAPIClient):
    # client options
    access_token: str

    def __init__(
        self,
        *,
        access_token: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncSpotted client instance.

        This automatically infers the `access_token` argument from the `SPOTIFY_ACCESS_TOKEN` environment variable if it is not provided.
        """
        if access_token is None:
            access_token = os.environ.get("SPOTIFY_ACCESS_TOKEN")
        if access_token is None:
            raise SpottedError(
                "The access_token client option must be set either by passing access_token to the client or by setting the SPOTIFY_ACCESS_TOKEN environment variable"
            )
        self.access_token = access_token

        if base_url is None:
            base_url = os.environ.get("SPOTTED_BASE_URL")
        if base_url is None:
            base_url = f"https://api.spotify.com/v1"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def albums(self) -> AsyncAlbumsResource:
        from .resources.albums import AsyncAlbumsResource

        return AsyncAlbumsResource(self)

    @cached_property
    def artists(self) -> AsyncArtistsResource:
        from .resources.artists import AsyncArtistsResource

        return AsyncArtistsResource(self)

    @cached_property
    def shows(self) -> AsyncShowsResource:
        from .resources.shows import AsyncShowsResource

        return AsyncShowsResource(self)

    @cached_property
    def episodes(self) -> AsyncEpisodesResource:
        from .resources.episodes import AsyncEpisodesResource

        return AsyncEpisodesResource(self)

    @cached_property
    def audiobooks(self) -> AsyncAudiobooksResource:
        from .resources.audiobooks import AsyncAudiobooksResource

        return AsyncAudiobooksResource(self)

    @cached_property
    def me(self) -> AsyncMeResource:
        from .resources.me import AsyncMeResource

        return AsyncMeResource(self)

    @cached_property
    def chapters(self) -> AsyncChaptersResource:
        from .resources.chapters import AsyncChaptersResource

        return AsyncChaptersResource(self)

    @cached_property
    def tracks(self) -> AsyncTracksResource:
        from .resources.tracks import AsyncTracksResource

        return AsyncTracksResource(self)

    @cached_property
    def search(self) -> AsyncSearchResource:
        from .resources.search import AsyncSearchResource

        return AsyncSearchResource(self)

    @cached_property
    def playlists(self) -> AsyncPlaylistsResource:
        from .resources.playlists import AsyncPlaylistsResource

        return AsyncPlaylistsResource(self)

    @cached_property
    def users(self) -> AsyncUsersResource:
        from .resources.users import AsyncUsersResource

        return AsyncUsersResource(self)

    @cached_property
    def browse(self) -> AsyncBrowseResource:
        from .resources.browse import AsyncBrowseResource

        return AsyncBrowseResource(self)

    @cached_property
    def audio_features(self) -> AsyncAudioFeaturesResource:
        from .resources.audio_features import AsyncAudioFeaturesResource

        return AsyncAudioFeaturesResource(self)

    @cached_property
    def audio_analysis(self) -> AsyncAudioAnalysisResource:
        from .resources.audio_analysis import AsyncAudioAnalysisResource

        return AsyncAudioAnalysisResource(self)

    @cached_property
    def recommendations(self) -> AsyncRecommendationsResource:
        from .resources.recommendations import AsyncRecommendationsResource

        return AsyncRecommendationsResource(self)

    @cached_property
    def markets(self) -> AsyncMarketsResource:
        from .resources.markets import AsyncMarketsResource

        return AsyncMarketsResource(self)

    @cached_property
    def with_raw_response(self) -> AsyncSpottedWithRawResponse:
        return AsyncSpottedWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSpottedWithStreamedResponse:
        return AsyncSpottedWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        access_token = self.access_token
        return {"Authorization": f"Bearer {access_token}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        access_token: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            access_token=access_token or self.access_token,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class SpottedWithRawResponse:
    _client: Spotted

    def __init__(self, client: Spotted) -> None:
        self._client = client

    @cached_property
    def albums(self) -> albums.AlbumsResourceWithRawResponse:
        from .resources.albums import AlbumsResourceWithRawResponse

        return AlbumsResourceWithRawResponse(self._client.albums)

    @cached_property
    def artists(self) -> artists.ArtistsResourceWithRawResponse:
        from .resources.artists import ArtistsResourceWithRawResponse

        return ArtistsResourceWithRawResponse(self._client.artists)

    @cached_property
    def shows(self) -> shows.ShowsResourceWithRawResponse:
        from .resources.shows import ShowsResourceWithRawResponse

        return ShowsResourceWithRawResponse(self._client.shows)

    @cached_property
    def episodes(self) -> episodes.EpisodesResourceWithRawResponse:
        from .resources.episodes import EpisodesResourceWithRawResponse

        return EpisodesResourceWithRawResponse(self._client.episodes)

    @cached_property
    def audiobooks(self) -> audiobooks.AudiobooksResourceWithRawResponse:
        from .resources.audiobooks import AudiobooksResourceWithRawResponse

        return AudiobooksResourceWithRawResponse(self._client.audiobooks)

    @cached_property
    def me(self) -> me.MeResourceWithRawResponse:
        from .resources.me import MeResourceWithRawResponse

        return MeResourceWithRawResponse(self._client.me)

    @cached_property
    def chapters(self) -> chapters.ChaptersResourceWithRawResponse:
        from .resources.chapters import ChaptersResourceWithRawResponse

        return ChaptersResourceWithRawResponse(self._client.chapters)

    @cached_property
    def tracks(self) -> tracks.TracksResourceWithRawResponse:
        from .resources.tracks import TracksResourceWithRawResponse

        return TracksResourceWithRawResponse(self._client.tracks)

    @cached_property
    def search(self) -> search.SearchResourceWithRawResponse:
        from .resources.search import SearchResourceWithRawResponse

        return SearchResourceWithRawResponse(self._client.search)

    @cached_property
    def playlists(self) -> playlists.PlaylistsResourceWithRawResponse:
        from .resources.playlists import PlaylistsResourceWithRawResponse

        return PlaylistsResourceWithRawResponse(self._client.playlists)

    @cached_property
    def users(self) -> users.UsersResourceWithRawResponse:
        from .resources.users import UsersResourceWithRawResponse

        return UsersResourceWithRawResponse(self._client.users)

    @cached_property
    def browse(self) -> browse.BrowseResourceWithRawResponse:
        from .resources.browse import BrowseResourceWithRawResponse

        return BrowseResourceWithRawResponse(self._client.browse)

    @cached_property
    def audio_features(self) -> audio_features.AudioFeaturesResourceWithRawResponse:
        from .resources.audio_features import AudioFeaturesResourceWithRawResponse

        return AudioFeaturesResourceWithRawResponse(self._client.audio_features)

    @cached_property
    def audio_analysis(self) -> audio_analysis.AudioAnalysisResourceWithRawResponse:
        from .resources.audio_analysis import AudioAnalysisResourceWithRawResponse

        return AudioAnalysisResourceWithRawResponse(self._client.audio_analysis)

    @cached_property
    def recommendations(self) -> recommendations.RecommendationsResourceWithRawResponse:
        from .resources.recommendations import RecommendationsResourceWithRawResponse

        return RecommendationsResourceWithRawResponse(self._client.recommendations)

    @cached_property
    def markets(self) -> markets.MarketsResourceWithRawResponse:
        from .resources.markets import MarketsResourceWithRawResponse

        return MarketsResourceWithRawResponse(self._client.markets)


class AsyncSpottedWithRawResponse:
    _client: AsyncSpotted

    def __init__(self, client: AsyncSpotted) -> None:
        self._client = client

    @cached_property
    def albums(self) -> albums.AsyncAlbumsResourceWithRawResponse:
        from .resources.albums import AsyncAlbumsResourceWithRawResponse

        return AsyncAlbumsResourceWithRawResponse(self._client.albums)

    @cached_property
    def artists(self) -> artists.AsyncArtistsResourceWithRawResponse:
        from .resources.artists import AsyncArtistsResourceWithRawResponse

        return AsyncArtistsResourceWithRawResponse(self._client.artists)

    @cached_property
    def shows(self) -> shows.AsyncShowsResourceWithRawResponse:
        from .resources.shows import AsyncShowsResourceWithRawResponse

        return AsyncShowsResourceWithRawResponse(self._client.shows)

    @cached_property
    def episodes(self) -> episodes.AsyncEpisodesResourceWithRawResponse:
        from .resources.episodes import AsyncEpisodesResourceWithRawResponse

        return AsyncEpisodesResourceWithRawResponse(self._client.episodes)

    @cached_property
    def audiobooks(self) -> audiobooks.AsyncAudiobooksResourceWithRawResponse:
        from .resources.audiobooks import AsyncAudiobooksResourceWithRawResponse

        return AsyncAudiobooksResourceWithRawResponse(self._client.audiobooks)

    @cached_property
    def me(self) -> me.AsyncMeResourceWithRawResponse:
        from .resources.me import AsyncMeResourceWithRawResponse

        return AsyncMeResourceWithRawResponse(self._client.me)

    @cached_property
    def chapters(self) -> chapters.AsyncChaptersResourceWithRawResponse:
        from .resources.chapters import AsyncChaptersResourceWithRawResponse

        return AsyncChaptersResourceWithRawResponse(self._client.chapters)

    @cached_property
    def tracks(self) -> tracks.AsyncTracksResourceWithRawResponse:
        from .resources.tracks import AsyncTracksResourceWithRawResponse

        return AsyncTracksResourceWithRawResponse(self._client.tracks)

    @cached_property
    def search(self) -> search.AsyncSearchResourceWithRawResponse:
        from .resources.search import AsyncSearchResourceWithRawResponse

        return AsyncSearchResourceWithRawResponse(self._client.search)

    @cached_property
    def playlists(self) -> playlists.AsyncPlaylistsResourceWithRawResponse:
        from .resources.playlists import AsyncPlaylistsResourceWithRawResponse

        return AsyncPlaylistsResourceWithRawResponse(self._client.playlists)

    @cached_property
    def users(self) -> users.AsyncUsersResourceWithRawResponse:
        from .resources.users import AsyncUsersResourceWithRawResponse

        return AsyncUsersResourceWithRawResponse(self._client.users)

    @cached_property
    def browse(self) -> browse.AsyncBrowseResourceWithRawResponse:
        from .resources.browse import AsyncBrowseResourceWithRawResponse

        return AsyncBrowseResourceWithRawResponse(self._client.browse)

    @cached_property
    def audio_features(self) -> audio_features.AsyncAudioFeaturesResourceWithRawResponse:
        from .resources.audio_features import AsyncAudioFeaturesResourceWithRawResponse

        return AsyncAudioFeaturesResourceWithRawResponse(self._client.audio_features)

    @cached_property
    def audio_analysis(self) -> audio_analysis.AsyncAudioAnalysisResourceWithRawResponse:
        from .resources.audio_analysis import AsyncAudioAnalysisResourceWithRawResponse

        return AsyncAudioAnalysisResourceWithRawResponse(self._client.audio_analysis)

    @cached_property
    def recommendations(self) -> recommendations.AsyncRecommendationsResourceWithRawResponse:
        from .resources.recommendations import AsyncRecommendationsResourceWithRawResponse

        return AsyncRecommendationsResourceWithRawResponse(self._client.recommendations)

    @cached_property
    def markets(self) -> markets.AsyncMarketsResourceWithRawResponse:
        from .resources.markets import AsyncMarketsResourceWithRawResponse

        return AsyncMarketsResourceWithRawResponse(self._client.markets)


class SpottedWithStreamedResponse:
    _client: Spotted

    def __init__(self, client: Spotted) -> None:
        self._client = client

    @cached_property
    def albums(self) -> albums.AlbumsResourceWithStreamingResponse:
        from .resources.albums import AlbumsResourceWithStreamingResponse

        return AlbumsResourceWithStreamingResponse(self._client.albums)

    @cached_property
    def artists(self) -> artists.ArtistsResourceWithStreamingResponse:
        from .resources.artists import ArtistsResourceWithStreamingResponse

        return ArtistsResourceWithStreamingResponse(self._client.artists)

    @cached_property
    def shows(self) -> shows.ShowsResourceWithStreamingResponse:
        from .resources.shows import ShowsResourceWithStreamingResponse

        return ShowsResourceWithStreamingResponse(self._client.shows)

    @cached_property
    def episodes(self) -> episodes.EpisodesResourceWithStreamingResponse:
        from .resources.episodes import EpisodesResourceWithStreamingResponse

        return EpisodesResourceWithStreamingResponse(self._client.episodes)

    @cached_property
    def audiobooks(self) -> audiobooks.AudiobooksResourceWithStreamingResponse:
        from .resources.audiobooks import AudiobooksResourceWithStreamingResponse

        return AudiobooksResourceWithStreamingResponse(self._client.audiobooks)

    @cached_property
    def me(self) -> me.MeResourceWithStreamingResponse:
        from .resources.me import MeResourceWithStreamingResponse

        return MeResourceWithStreamingResponse(self._client.me)

    @cached_property
    def chapters(self) -> chapters.ChaptersResourceWithStreamingResponse:
        from .resources.chapters import ChaptersResourceWithStreamingResponse

        return ChaptersResourceWithStreamingResponse(self._client.chapters)

    @cached_property
    def tracks(self) -> tracks.TracksResourceWithStreamingResponse:
        from .resources.tracks import TracksResourceWithStreamingResponse

        return TracksResourceWithStreamingResponse(self._client.tracks)

    @cached_property
    def search(self) -> search.SearchResourceWithStreamingResponse:
        from .resources.search import SearchResourceWithStreamingResponse

        return SearchResourceWithStreamingResponse(self._client.search)

    @cached_property
    def playlists(self) -> playlists.PlaylistsResourceWithStreamingResponse:
        from .resources.playlists import PlaylistsResourceWithStreamingResponse

        return PlaylistsResourceWithStreamingResponse(self._client.playlists)

    @cached_property
    def users(self) -> users.UsersResourceWithStreamingResponse:
        from .resources.users import UsersResourceWithStreamingResponse

        return UsersResourceWithStreamingResponse(self._client.users)

    @cached_property
    def browse(self) -> browse.BrowseResourceWithStreamingResponse:
        from .resources.browse import BrowseResourceWithStreamingResponse

        return BrowseResourceWithStreamingResponse(self._client.browse)

    @cached_property
    def audio_features(self) -> audio_features.AudioFeaturesResourceWithStreamingResponse:
        from .resources.audio_features import AudioFeaturesResourceWithStreamingResponse

        return AudioFeaturesResourceWithStreamingResponse(self._client.audio_features)

    @cached_property
    def audio_analysis(self) -> audio_analysis.AudioAnalysisResourceWithStreamingResponse:
        from .resources.audio_analysis import AudioAnalysisResourceWithStreamingResponse

        return AudioAnalysisResourceWithStreamingResponse(self._client.audio_analysis)

    @cached_property
    def recommendations(self) -> recommendations.RecommendationsResourceWithStreamingResponse:
        from .resources.recommendations import RecommendationsResourceWithStreamingResponse

        return RecommendationsResourceWithStreamingResponse(self._client.recommendations)

    @cached_property
    def markets(self) -> markets.MarketsResourceWithStreamingResponse:
        from .resources.markets import MarketsResourceWithStreamingResponse

        return MarketsResourceWithStreamingResponse(self._client.markets)


class AsyncSpottedWithStreamedResponse:
    _client: AsyncSpotted

    def __init__(self, client: AsyncSpotted) -> None:
        self._client = client

    @cached_property
    def albums(self) -> albums.AsyncAlbumsResourceWithStreamingResponse:
        from .resources.albums import AsyncAlbumsResourceWithStreamingResponse

        return AsyncAlbumsResourceWithStreamingResponse(self._client.albums)

    @cached_property
    def artists(self) -> artists.AsyncArtistsResourceWithStreamingResponse:
        from .resources.artists import AsyncArtistsResourceWithStreamingResponse

        return AsyncArtistsResourceWithStreamingResponse(self._client.artists)

    @cached_property
    def shows(self) -> shows.AsyncShowsResourceWithStreamingResponse:
        from .resources.shows import AsyncShowsResourceWithStreamingResponse

        return AsyncShowsResourceWithStreamingResponse(self._client.shows)

    @cached_property
    def episodes(self) -> episodes.AsyncEpisodesResourceWithStreamingResponse:
        from .resources.episodes import AsyncEpisodesResourceWithStreamingResponse

        return AsyncEpisodesResourceWithStreamingResponse(self._client.episodes)

    @cached_property
    def audiobooks(self) -> audiobooks.AsyncAudiobooksResourceWithStreamingResponse:
        from .resources.audiobooks import AsyncAudiobooksResourceWithStreamingResponse

        return AsyncAudiobooksResourceWithStreamingResponse(self._client.audiobooks)

    @cached_property
    def me(self) -> me.AsyncMeResourceWithStreamingResponse:
        from .resources.me import AsyncMeResourceWithStreamingResponse

        return AsyncMeResourceWithStreamingResponse(self._client.me)

    @cached_property
    def chapters(self) -> chapters.AsyncChaptersResourceWithStreamingResponse:
        from .resources.chapters import AsyncChaptersResourceWithStreamingResponse

        return AsyncChaptersResourceWithStreamingResponse(self._client.chapters)

    @cached_property
    def tracks(self) -> tracks.AsyncTracksResourceWithStreamingResponse:
        from .resources.tracks import AsyncTracksResourceWithStreamingResponse

        return AsyncTracksResourceWithStreamingResponse(self._client.tracks)

    @cached_property
    def search(self) -> search.AsyncSearchResourceWithStreamingResponse:
        from .resources.search import AsyncSearchResourceWithStreamingResponse

        return AsyncSearchResourceWithStreamingResponse(self._client.search)

    @cached_property
    def playlists(self) -> playlists.AsyncPlaylistsResourceWithStreamingResponse:
        from .resources.playlists import AsyncPlaylistsResourceWithStreamingResponse

        return AsyncPlaylistsResourceWithStreamingResponse(self._client.playlists)

    @cached_property
    def users(self) -> users.AsyncUsersResourceWithStreamingResponse:
        from .resources.users import AsyncUsersResourceWithStreamingResponse

        return AsyncUsersResourceWithStreamingResponse(self._client.users)

    @cached_property
    def browse(self) -> browse.AsyncBrowseResourceWithStreamingResponse:
        from .resources.browse import AsyncBrowseResourceWithStreamingResponse

        return AsyncBrowseResourceWithStreamingResponse(self._client.browse)

    @cached_property
    def audio_features(self) -> audio_features.AsyncAudioFeaturesResourceWithStreamingResponse:
        from .resources.audio_features import AsyncAudioFeaturesResourceWithStreamingResponse

        return AsyncAudioFeaturesResourceWithStreamingResponse(self._client.audio_features)

    @cached_property
    def audio_analysis(self) -> audio_analysis.AsyncAudioAnalysisResourceWithStreamingResponse:
        from .resources.audio_analysis import AsyncAudioAnalysisResourceWithStreamingResponse

        return AsyncAudioAnalysisResourceWithStreamingResponse(self._client.audio_analysis)

    @cached_property
    def recommendations(self) -> recommendations.AsyncRecommendationsResourceWithStreamingResponse:
        from .resources.recommendations import AsyncRecommendationsResourceWithStreamingResponse

        return AsyncRecommendationsResourceWithStreamingResponse(self._client.recommendations)

    @cached_property
    def markets(self) -> markets.AsyncMarketsResourceWithStreamingResponse:
        from .resources.markets import AsyncMarketsResourceWithStreamingResponse

        return AsyncMarketsResourceWithStreamingResponse(self._client.markets)


Client = Spotted

AsyncClient = AsyncSpotted



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import typing_extensions

import httpx

from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.playlists import follower_check_params, follower_follow_params
from ...types.playlists.follower_check_response import FollowerCheckResponse

__all__ = ["FollowersResource", "AsyncFollowersResource"]


class FollowersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> FollowersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return FollowersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FollowersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return FollowersResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    def check(
        self,
        playlist_id: str,
        *,
        ids: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FollowerCheckResponse:
        """
        Check to see if the current user is following a specified playlist.

        **Note:** This endpoint is deprecated. Use
        [Check User's Saved Items](/documentation/web-api/reference/check-library-contains)
        instead.

        Args:
          playlist_id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              playlist.

          ids: **Deprecated** A single item list containing current user's
              [Spotify Username](/documentation/web-api/concepts/spotify-uris-ids). Maximum: 1
              id.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not playlist_id:
            raise ValueError(f"Expected a non-empty value for `playlist_id` but received {playlist_id!r}")
        return self._get(
            f"/playlists/{playlist_id}/followers/contains",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"ids": ids}, follower_check_params.FollowerCheckParams),
            ),
            cast_to=FollowerCheckResponse,
        )

    @typing_extensions.deprecated("deprecated")
    def follow(
        self,
        playlist_id: str,
        *,
        published: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Add the current user as a follower of a playlist.

        **Note:** This endpoint is deprecated. Use
        [Save Items to Library](/documentation/web-api/reference/save-library-items)
        instead.

        Args:
          playlist_id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              playlist.

          published: The playlist's public/private status (if it should be added to the user's
              profile or not): `true` the playlist will be public, `false` the playlist will
              be private, `null` the playlist status is not relevant. For more about
              public/private status, see
              [Working with Playlists](/documentation/web-api/concepts/playlists)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not playlist_id:
            raise ValueError(f"Expected a non-empty value for `playlist_id` but received {playlist_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._put(
            f"/playlists/{playlist_id}/followers",
            body=maybe_transform({"published": published}, follower_follow_params.FollowerFollowParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    @typing_extensions.deprecated("deprecated")
    def unfollow(
        self,
        playlist_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Remove the current user as a follower of a playlist.

        **Note:** This endpoint is deprecated. Use
        [Remove Items from Library](/documentation/web-api/reference/remove-library-items)
        instead.

        Args:
          playlist_id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              playlist.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not playlist_id:
            raise ValueError(f"Expected a non-empty value for `playlist_id` but received {playlist_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/playlists/{playlist_id}/followers",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncFollowersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncFollowersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return AsyncFollowersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFollowersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return AsyncFollowersResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    async def check(
        self,
        playlist_id: str,
        *,
        ids: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FollowerCheckResponse:
        """
        Check to see if the current user is following a specified playlist.

        **Note:** This endpoint is deprecated. Use
        [Check User's Saved Items](/documentation/web-api/reference/check-library-contains)
        instead.

        Args:
          playlist_id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              playlist.

          ids: **Deprecated** A single item list containing current user's
              [Spotify Username](/documentation/web-api/concepts/spotify-uris-ids). Maximum: 1
              id.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not playlist_id:
            raise ValueError(f"Expected a non-empty value for `playlist_id` but received {playlist_id!r}")
        return await self._get(
            f"/playlists/{playlist_id}/followers/contains",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"ids": ids}, follower_check_params.FollowerCheckParams),
            ),
            cast_to=FollowerCheckResponse,
        )

    @typing_extensions.deprecated("deprecated")
    async def follow(
        self,
        playlist_id: str,
        *,
        published: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Add the current user as a follower of a playlist.

        **Note:** This endpoint is deprecated. Use
        [Save Items to Library](/documentation/web-api/reference/save-library-items)
        instead.

        Args:
          playlist_id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              playlist.

          published: The playlist's public/private status (if it should be added to the user's
              profile or not): `true` the playlist will be public, `false` the playlist will
              be private, `null` the playlist status is not relevant. For more about
              public/private status, see
              [Working with Playlists](/documentation/web-api/concepts/playlists)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not playlist_id:
            raise ValueError(f"Expected a non-empty value for `playlist_id` but received {playlist_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._put(
            f"/playlists/{playlist_id}/followers",
            body=await async_maybe_transform({"published": published}, follower_follow_params.FollowerFollowParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    @typing_extensions.deprecated("deprecated")
    async def unfollow(
        self,
        playlist_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Remove the current user as a follower of a playlist.

        **Note:** This endpoint is deprecated. Use
        [Remove Items from Library](/documentation/web-api/reference/remove-library-items)
        instead.

        Args:
          playlist_id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              playlist.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not playlist_id:
            raise ValueError(f"Expected a non-empty value for `playlist_id` but received {playlist_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/playlists/{playlist_id}/followers",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class FollowersResourceWithRawResponse:
    def __init__(self, followers: FollowersResource) -> None:
        self._followers = followers

        self.check = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                followers.check,  # pyright: ignore[reportDeprecated],
            )
        )
        self.follow = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                followers.follow,  # pyright: ignore[reportDeprecated],
            )
        )
        self.unfollow = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                followers.unfollow,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncFollowersResourceWithRawResponse:
    def __init__(self, followers: AsyncFollowersResource) -> None:
        self._followers = followers

        self.check = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                followers.check,  # pyright: ignore[reportDeprecated],
            )
        )
        self.follow = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                followers.follow,  # pyright: ignore[reportDeprecated],
            )
        )
        self.unfollow = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                followers.unfollow,  # pyright: ignore[reportDeprecated],
            )
        )


class FollowersResourceWithStreamingResponse:
    def __init__(self, followers: FollowersResource) -> None:
        self._followers = followers

        self.check = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                followers.check,  # pyright: ignore[reportDeprecated],
            )
        )
        self.follow = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                followers.follow,  # pyright: ignore[reportDeprecated],
            )
        )
        self.unfollow = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                followers.unfollow,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncFollowersResourceWithStreamingResponse:
    def __init__(self, followers: AsyncFollowersResource) -> None:
        self._followers = followers

        self.check = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                followers.check,  # pyright: ignore[reportDeprecated],
            )
        )
        self.follow = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                followers.follow,  # pyright: ignore[reportDeprecated],
            )
        )
        self.unfollow = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                followers.unfollow,  # pyright: ignore[reportDeprecated],
            )
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from ..._types import SequenceNotStr

__all__ = ["EpisodeRemoveParams"]


class EpisodeRemoveParams(TypedDict, total=False):
    ids: SequenceNotStr[str]
    """
    A JSON array of the
    [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids). <br/>A maximum
    of 50 items can be specified in one request. _**Note**: if the `ids` parameter
    is present in the query string, any IDs listed here in the body will be
    ignored._
    """

    published: bool
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .device_object import DeviceObject as DeviceObject
from .context_object import ContextObject as ContextObject
from .show_list_params import ShowListParams as ShowListParams
from .show_save_params import ShowSaveParams as ShowSaveParams
from .album_list_params import AlbumListParams as AlbumListParams
from .album_save_params import AlbumSaveParams as AlbumSaveParams
from .show_check_params import ShowCheckParams as ShowCheckParams
from .track_list_params import TrackListParams as TrackListParams
from .track_save_params import TrackSaveParams as TrackSaveParams
from .album_check_params import AlbumCheckParams as AlbumCheckParams
from .show_list_response import ShowListResponse as ShowListResponse
from .show_remove_params import ShowRemoveParams as ShowRemoveParams
from .track_check_params import TrackCheckParams as TrackCheckParams
from .album_list_response import AlbumListResponse as AlbumListResponse
from .album_remove_params import AlbumRemoveParams as AlbumRemoveParams
from .episode_list_params import EpisodeListParams as EpisodeListParams
from .episode_save_params import EpisodeSaveParams as EpisodeSaveParams
from .show_check_response import ShowCheckResponse as ShowCheckResponse
from .track_list_response import TrackListResponse as TrackListResponse
from .track_remove_params import TrackRemoveParams as TrackRemoveParams
from .album_check_response import AlbumCheckResponse as AlbumCheckResponse
from .episode_check_params import EpisodeCheckParams as EpisodeCheckParams
from .playlist_list_params import PlaylistListParams as PlaylistListParams
from .track_check_response import TrackCheckResponse as TrackCheckResponse
from .audiobook_list_params import AudiobookListParams as AudiobookListParams
from .audiobook_save_params import AudiobookSaveParams as AudiobookSaveParams
from .episode_list_response import EpisodeListResponse as EpisodeListResponse
from .episode_remove_params import EpisodeRemoveParams as EpisodeRemoveParams
from .audiobook_check_params import AudiobookCheckParams as AudiobookCheckParams
from .episode_check_response import EpisodeCheckResponse as EpisodeCheckResponse
from .following_check_params import FollowingCheckParams as FollowingCheckParams
from .player_transfer_params import PlayerTransferParams as PlayerTransferParams
from .audiobook_list_response import AudiobookListResponse as AudiobookListResponse
from .audiobook_remove_params import AudiobookRemoveParams as AudiobookRemoveParams
from .following_follow_params import FollowingFollowParams as FollowingFollowParams
from .player_get_state_params import PlayerGetStateParams as PlayerGetStateParams
from .player_skip_next_params import PlayerSkipNextParams as PlayerSkipNextParams
from .audiobook_check_response import AudiobookCheckResponse as AudiobookCheckResponse
from .following_check_response import FollowingCheckResponse as FollowingCheckResponse
from .player_set_volume_params import PlayerSetVolumeParams as PlayerSetVolumeParams
from .following_unfollow_params import FollowingUnfollowParams as FollowingUnfollowParams
from .player_get_state_response import PlayerGetStateResponse as PlayerGetStateResponse
from .top_list_top_tracks_params import TopListTopTracksParams as TopListTopTracksParams
from .player_get_devices_response import PlayerGetDevicesResponse as PlayerGetDevicesResponse
from .player_skip_previous_params import PlayerSkipPreviousParams as PlayerSkipPreviousParams
from .top_list_top_artists_params import TopListTopArtistsParams as TopListTopArtistsParams
from .player_pause_playback_params import PlayerPausePlaybackParams as PlayerPausePlaybackParams
from .player_start_playback_params import PlayerStartPlaybackParams as PlayerStartPlaybackParams
from .player_toggle_shuffle_params import PlayerToggleShuffleParams as PlayerToggleShuffleParams
from .player_set_repeat_mode_params import PlayerSetRepeatModeParams as PlayerSetRepeatModeParams
from .following_bulk_retrieve_params import FollowingBulkRetrieveParams as FollowingBulkRetrieveParams
from .player_seek_to_position_params import PlayerSeekToPositionParams as PlayerSeekToPositionParams
from .following_bulk_retrieve_response import FollowingBulkRetrieveResponse as FollowingBulkRetrieveResponse
from .player_list_recently_played_params import PlayerListRecentlyPlayedParams as PlayerListRecentlyPlayedParams
from .player_get_currently_playing_params import PlayerGetCurrentlyPlayingParams as PlayerGetCurrentlyPlayingParams
from .player_list_recently_played_response import PlayerListRecentlyPlayedResponse as PlayerListRecentlyPlayedResponse
from .player_get_currently_playing_response import (
    PlayerGetCurrentlyPlayingResponse as PlayerGetCurrentlyPlayingResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from spotted import Spotted, AsyncSpotted
from tests.utils import assert_matches_type
from spotted.types import PlaylistRetrieveResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPlaylists:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Spotted) -> None:
        playlist = client.playlists.retrieve(
            playlist_id="3cEYpjA9oz9GiPac4AsH4n",
        )
        assert_matches_type(PlaylistRetrieveResponse, playlist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve_with_all_params(self, client: Spotted) -> None:
        playlist = client.playlists.retrieve(
            playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            additional_types="additional_types",
            fields="items(added_by.id,track(name,href,album(name,href)))",
            market="ES",
        )
        assert_matches_type(PlaylistRetrieveResponse, playlist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Spotted) -> None:
        response = client.playlists.with_raw_response.retrieve(
            playlist_id="3cEYpjA9oz9GiPac4AsH4n",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        playlist = response.parse()
        assert_matches_type(PlaylistRetrieveResponse, playlist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Spotted) -> None:
        with client.playlists.with_streaming_response.retrieve(
            playlist_id="3cEYpjA9oz9GiPac4AsH4n",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            playlist = response.parse()
            assert_matches_type(PlaylistRetrieveResponse, playlist, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Spotted) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `playlist_id` but received ''"):
            client.playlists.with_raw_response.retrieve(
                playlist_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update(self, client: Spotted) -> None:
        playlist = client.playlists.update(
            playlist_id="3cEYpjA9oz9GiPac4AsH4n",
        )
        assert playlist is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Spotted) -> None:
        playlist = client.playlists.update(
            playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            collaborative=True,
            description="Updated playlist description",
            name="Updated Playlist Name",
            published=True,
        )
        assert playlist is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Spotted) -> None:
        response = client.playlists.with_raw_response.update(
            playlist_id="3cEYpjA9oz9GiPac4AsH4n",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        playlist = response.parse()
        assert playlist is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Spotted) -> None:
        with client.playlists.with_streaming_response.update(
            playlist_id="3cEYpjA9oz9GiPac4AsH4n",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            playlist = response.parse()
            assert playlist is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Spotted) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `playlist_id` but received ''"):
            client.playlists.with_raw_response.update(
                playlist_id="",
            )


class TestAsyncPlaylists:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSpotted) -> None:
        playlist = await async_client.playlists.retrieve(
            playlist_id="3cEYpjA9oz9GiPac4AsH4n",
        )
        assert_matches_type(PlaylistRetrieveResponse, playlist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncSpotted) -> None:
        playlist = await async_client.playlists.retrieve(
            playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            additional_types="additional_types",
            fields="items(added_by.id,track(name,href,album(name,href)))",
            market="ES",
        )
        assert_matches_type(PlaylistRetrieveResponse, playlist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSpotted) -> None:
        response = await async_client.playlists.with_raw_response.retrieve(
            playlist_id="3cEYpjA9oz9GiPac4AsH4n",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        playlist = await response.parse()
        assert_matches_type(PlaylistRetrieveResponse, playlist, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSpotted) -> None:
        async with async_client.playlists.with_streaming_response.retrieve(
            playlist_id="3cEYpjA9oz9GiPac4AsH4n",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            playlist = await response.parse()
            assert_matches_type(PlaylistRetrieveResponse, playlist, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSpotted) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `playlist_id` but received ''"):
            await async_client.playlists.with_raw_response.retrieve(
                playlist_id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncSpotted) -> None:
        playlist = await async_client.playlists.update(
            playlist_id="3cEYpjA9oz9GiPac4AsH4n",
        )
        assert playlist is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncSpotted) -> None:
        playlist = await async_client.playlists.update(
            playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            collaborative=True,
            description="Updated playlist description",
            name="Updated Playlist Name",
            published=True,
        )
        assert playlist is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncSpotted) -> None:
        response = await async_client.playlists.with_raw_response.update(
            playlist_id="3cEYpjA9oz9GiPac4AsH4n",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        playlist = await response.parse()
        assert playlist is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncSpotted) -> None:
        async with async_client.playlists.with_streaming_response.update(
            playlist_id="3cEYpjA9oz9GiPac4AsH4n",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            playlist = await response.parse()
            assert playlist is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncSpotted) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `playlist_id` but received ''"):
            await async_client.playlists.with_raw_response.update(
                playlist_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .shared.image_object import ImageObject
from .shared.followers_object import FollowersObject
from .shared.external_url_object import ExternalURLObject

__all__ = ["MeRetrieveResponse", "ExplicitContent"]


class ExplicitContent(BaseModel):
    """The user's explicit content settings.

    _This field is only available when the current user has granted access to the [user-read-private](/documentation/web-api/concepts/scopes/#list-of-scopes) scope._
    """

    filter_enabled: Optional[bool] = None
    """When `true`, indicates that explicit content should not be played."""

    filter_locked: Optional[bool] = None
    """
    When `true`, indicates that the explicit content setting is locked and can't be
    changed by the user.
    """

    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """


class MeRetrieveResponse(BaseModel):
    id: Optional[str] = None
    """
    The [Spotify user ID](/documentation/web-api/concepts/spotify-uris-ids) for the
    user.
    """

    country: Optional[str] = None
    """The country of the user, as set in the user's account profile.

    An
    [ISO 3166-1 alpha-2 country code](http://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
    _This field is only available when the current user has granted access to the
    [user-read-private](/documentation/web-api/concepts/scopes/#list-of-scopes)
    scope._
    """

    display_name: Optional[str] = None
    """The name displayed on the user's profile. `null` if not available."""

    email: Optional[str] = None
    """The user's email address, as entered by the user when creating their account.

    _**Important!** This email address is unverified; there is no proof that it
    actually belongs to the user._ _This field is only available when the current
    user has granted access to the
    [user-read-email](/documentation/web-api/concepts/scopes/#list-of-scopes)
    scope._
    """

    explicit_content: Optional[ExplicitContent] = None
    """The user's explicit content settings.

    _This field is only available when the current user has granted access to the
    [user-read-private](/documentation/web-api/concepts/scopes/#list-of-scopes)
    scope._
    """

    external_urls: Optional[ExternalURLObject] = None
    """Known external URLs for this user."""

    followers: Optional[FollowersObject] = None
    """Information about the followers of the user."""

    href: Optional[str] = None
    """A link to the Web API endpoint for this user."""

    images: Optional[List[ImageObject]] = None
    """The user's profile image."""

    product: Optional[str] = None
    """The user's Spotify subscription level: "premium", "free", etc.

    (The subscription level "open" can be considered the same as "free".) _This
    field is only available when the current user has granted access to the
    [user-read-private](/documentation/web-api/concepts/scopes/#list-of-scopes)
    scope._
    """

    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """

    type: Optional[str] = None
    """The object type: "user" """

    uri: Optional[str] = None
    """
    The [Spotify URI](/documentation/web-api/concepts/spotify-uris-ids) for the
    user.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel
from .device_object import DeviceObject

__all__ = ["PlayerGetDevicesResponse"]


class PlayerGetDevicesResponse(BaseModel):
    devices: List[DeviceObject]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import httpx
import pytest
from respx import MockRouter

from spotted import Spotted, AsyncSpotted
from tests.utils import assert_matches_type
from spotted._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
)
from spotted.types.playlists import ImageListResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestImages:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_update(self, client: Spotted, respx_mock: MockRouter) -> None:
        respx_mock.put("/playlists/3cEYpjA9oz9GiPac4AsH4n/images").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        image = client.playlists.images.update(
            playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            body=b"Example data",
        )
        assert image.is_closed
        assert image.json() == {"foo": "bar"}
        assert cast(Any, image.is_closed) is True
        assert isinstance(image, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_raw_response_update(self, client: Spotted, respx_mock: MockRouter) -> None:
        respx_mock.put("/playlists/3cEYpjA9oz9GiPac4AsH4n/images").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )

        image = client.playlists.images.with_raw_response.update(
            playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            body=b"Example data",
        )

        assert image.is_closed is True
        assert image.http_request.headers.get("X-Stainless-Lang") == "python"
        assert image.json() == {"foo": "bar"}
        assert isinstance(image, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_streaming_response_update(self, client: Spotted, respx_mock: MockRouter) -> None:
        respx_mock.put("/playlists/3cEYpjA9oz9GiPac4AsH4n/images").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        with client.playlists.images.with_streaming_response.update(
            playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            body=b"Example data",
        ) as image:
            assert not image.is_closed
            assert image.http_request.headers.get("X-Stainless-Lang") == "python"

            assert image.json() == {"foo": "bar"}
            assert cast(Any, image.is_closed) is True
            assert isinstance(image, StreamedBinaryAPIResponse)

        assert cast(Any, image.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_path_params_update(self, client: Spotted) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `playlist_id` but received ''"):
            client.playlists.images.with_raw_response.update(
                playlist_id="",
                body=b"Example data",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Spotted) -> None:
        image = client.playlists.images.list(
            "3cEYpjA9oz9GiPac4AsH4n",
        )
        assert_matches_type(ImageListResponse, image, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Spotted) -> None:
        response = client.playlists.images.with_raw_response.list(
            "3cEYpjA9oz9GiPac4AsH4n",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        image = response.parse()
        assert_matches_type(ImageListResponse, image, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Spotted) -> None:
        with client.playlists.images.with_streaming_response.list(
            "3cEYpjA9oz9GiPac4AsH4n",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            image = response.parse()
            assert_matches_type(ImageListResponse, image, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_list(self, client: Spotted) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `playlist_id` but received ''"):
            client.playlists.images.with_raw_response.list(
                "",
            )


class TestAsyncImages:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_update(self, async_client: AsyncSpotted, respx_mock: MockRouter) -> None:
        respx_mock.put("/playlists/3cEYpjA9oz9GiPac4AsH4n/images").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        image = await async_client.playlists.images.update(
            playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            body=b"Example data",
        )
        assert image.is_closed
        assert await image.json() == {"foo": "bar"}
        assert cast(Any, image.is_closed) is True
        assert isinstance(image, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_raw_response_update(self, async_client: AsyncSpotted, respx_mock: MockRouter) -> None:
        respx_mock.put("/playlists/3cEYpjA9oz9GiPac4AsH4n/images").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )

        image = await async_client.playlists.images.with_raw_response.update(
            playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            body=b"Example data",
        )

        assert image.is_closed is True
        assert image.http_request.headers.get("X-Stainless-Lang") == "python"
        assert await image.json() == {"foo": "bar"}
        assert isinstance(image, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_streaming_response_update(self, async_client: AsyncSpotted, respx_mock: MockRouter) -> None:
        respx_mock.put("/playlists/3cEYpjA9oz9GiPac4AsH4n/images").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        async with async_client.playlists.images.with_streaming_response.update(
            playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            body=b"Example data",
        ) as image:
            assert not image.is_closed
            assert image.http_request.headers.get("X-Stainless-Lang") == "python"

            assert await image.json() == {"foo": "bar"}
            assert cast(Any, image.is_closed) is True
            assert isinstance(image, AsyncStreamedBinaryAPIResponse)

        assert cast(Any, image.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_path_params_update(self, async_client: AsyncSpotted) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `playlist_id` but received ''"):
            await async_client.playlists.images.with_raw_response.update(
                playlist_id="",
                body=b"Example data",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncSpotted) -> None:
        image = await async_client.playlists.images.list(
            "3cEYpjA9oz9GiPac4AsH4n",
        )
        assert_matches_type(ImageListResponse, image, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSpotted) -> None:
        response = await async_client.playlists.images.with_raw_response.list(
            "3cEYpjA9oz9GiPac4AsH4n",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        image = await response.parse()
        assert_matches_type(ImageListResponse, image, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSpotted) -> None:
        async with async_client.playlists.images.with_streaming_response.list(
            "3cEYpjA9oz9GiPac4AsH4n",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            image = await response.parse()
            assert_matches_type(ImageListResponse, image, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_list(self, async_client: AsyncSpotted) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `playlist_id` but received ''"):
            await async_client.playlists.images.with_raw_response.list(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["AlbumRestrictionObject"]


class AlbumRestrictionObject(BaseModel):
    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """

    reason: Optional[Literal["market", "product", "explicit"]] = None
    """The reason for the restriction.

    Albums may be restricted if the content is not available in a given market, to
    the user's subscription type, or when the user's account is set to not play
    explicit content. Additional reasons may be added in the future.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import typing_extensions
from typing import Iterable

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursorURLPage, AsyncCursorURLPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.playlists import track_add_params, track_list_params, track_remove_params, track_update_params
from ...types.playlists.track_add_response import TrackAddResponse
from ...types.shared.playlist_track_object import PlaylistTrackObject
from ...types.playlists.track_remove_response import TrackRemoveResponse
from ...types.playlists.track_update_response import TrackUpdateResponse

__all__ = ["TracksResource", "AsyncTracksResource"]


class TracksResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TracksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return TracksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TracksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return TracksResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    def update(
        self,
        playlist_id: str,
        *,
        insert_before: int | Omit = omit,
        published: bool | Omit = omit,
        range_length: int | Omit = omit,
        range_start: int | Omit = omit,
        snapshot_id: str | Omit = omit,
        uris: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TrackUpdateResponse:
        """
        **Deprecated:** Use
        [Update Playlist Items](/documentation/web-api/reference/reorder-or-replace-playlists-items)
        instead.

        Either reorder or replace items in a playlist depending on the request's
        parameters. To reorder items, include `range_start`, `insert_before`,
        `range_length` and `snapshot_id` in the request's body. To replace items,
        include `uris` as either a query parameter or in the request's body. Replacing
        items in a playlist will overwrite its existing items. This operation can be
        used for replacing or clearing items in a playlist. <br/> **Note**: Replace and
        reorder are mutually exclusive operations which share the same endpoint, but
        have different parameters. These operations can't be applied together in a
        single request.

        Args:
          playlist_id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              playlist.

          insert_before: The position where the items should be inserted.<br/>To reorder the items to the
              end of the playlist, simply set _insert_before_ to the position after the last
              item.<br/>Examples:<br/>To reorder the first item to the last position in a
              playlist with 10 items, set _range_start_ to 0, and _insert_before_
              to 10.<br/>To reorder the last item in a playlist with 10 items to the start of
              the playlist, set _range_start_ to 9, and _insert_before_ to 0.

          published: The playlist's public/private status (if it should be added to the user's
              profile or not): `true` the playlist will be public, `false` the playlist will
              be private, `null` the playlist status is not relevant. For more about
              public/private status, see
              [Working with Playlists](/documentation/web-api/concepts/playlists)

          range_length: The amount of items to be reordered. Defaults to 1 if not set.<br/>The range of
              items to be reordered begins from the _range_start_ position, and includes the
              _range_length_ subsequent items.<br/>Example:<br/>To move the items at index
              9-10 to the start of the playlist, _range_start_ is set to 9, and _range_length_
              is set to 2.

          range_start: The position of the first item to be reordered.

          snapshot_id: The playlist's snapshot ID against which you want to make the changes.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not playlist_id:
            raise ValueError(f"Expected a non-empty value for `playlist_id` but received {playlist_id!r}")
        return self._put(
            f"/playlists/{playlist_id}/tracks",
            body=maybe_transform(
                {
                    "insert_before": insert_before,
                    "published": published,
                    "range_length": range_length,
                    "range_start": range_start,
                    "snapshot_id": snapshot_id,
                    "uris": uris,
                },
                track_update_params.TrackUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TrackUpdateResponse,
        )

    @typing_extensions.deprecated("deprecated")
    def list(
        self,
        playlist_id: str,
        *,
        additional_types: str | Omit = omit,
        fields: str | Omit = omit,
        limit: int | Omit = omit,
        market: str | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorURLPage[PlaylistTrackObject]:
        """
        **Deprecated:** Use
        [Get Playlist Items](/documentation/web-api/reference/get-playlists-items)
        instead.

        Get full details of the items of a playlist owned by a Spotify user.

        Args:
          playlist_id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              playlist.

          additional_types: A comma-separated list of item types that your client supports besides the
              default `track` type. Valid types are: `track` and `episode`.<br/> _**Note**:
              This parameter was introduced to allow existing clients to maintain their
              current behaviour and might be deprecated in the future._<br/> In addition to
              providing this parameter, make sure that your client properly handles cases of
              new types in the future by checking against the `type` field of each object.

          fields: Filters for the query: a comma-separated list of the fields to return. If
              omitted, all fields are returned. For example, to get just the total number of
              items and the request limit:<br/>`fields=total,limit`<br/>A dot separator can be
              used to specify non-reoccurring fields, while parentheses can be used to specify
              reoccurring fields within objects. For example, to get just the added date and
              user ID of the adder:<br/>`fields=items(added_at,added_by.id)`<br/>Use multiple
              parentheses to drill down into nested objects, for
              example:<br/>`fields=items(track(name,href,album(name,href)))`<br/>Fields can be
              excluded by prefixing them with an exclamation mark, for
              example:<br/>`fields=items.track.album(!external_urls,images)`

          limit: The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 100.

          market: An
              [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
              If a country code is specified, only content that is available in that market
              will be returned.<br/> If a valid user access token is specified in the request
              header, the country associated with the user account will take priority over
              this parameter.<br/> _**Note**: If neither market or user country are provided,
              the content is considered unavailable for the client._<br/> Users can view the
              country that is associated with their account in the
              [account settings](https://www.spotify.com/account/overview/).

          offset: The index of the first item to return. Default: 0 (the first item). Use with
              limit to get the next set of items.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not playlist_id:
            raise ValueError(f"Expected a non-empty value for `playlist_id` but received {playlist_id!r}")
        return self._get_api_list(
            f"/playlists/{playlist_id}/tracks",
            page=SyncCursorURLPage[PlaylistTrackObject],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "additional_types": additional_types,
                        "fields": fields,
                        "limit": limit,
                        "market": market,
                        "offset": offset,
                    },
                    track_list_params.TrackListParams,
                ),
            ),
            model=PlaylistTrackObject,
        )

    @typing_extensions.deprecated("deprecated")
    def add(
        self,
        playlist_id: str,
        *,
        position: int | Omit = omit,
        published: bool | Omit = omit,
        uris: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TrackAddResponse:
        """
        **Deprecated:** Use
        [Add Items to Playlist](/documentation/web-api/reference/add-items-to-playlist)
        instead.

        Add one or more items to a user's playlist.

        Args:
          playlist_id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              playlist.

          position: The position to insert the items, a zero-based index. For example, to insert the
              items in the first position: `position=0` ; to insert the items in the third
              position: `position=2`. If omitted, the items will be appended to the playlist.
              Items are added in the order they appear in the uris array. For example:
              `{"uris": ["spotify:track:4iV5W9uYEdYUVa79Axb7Rh","spotify:track:1301WleyT98MSxVHPZCA6M"], "position": 3}`

          published: The playlist's public/private status (if it should be added to the user's
              profile or not): `true` the playlist will be public, `false` the playlist will
              be private, `null` the playlist status is not relevant. For more about
              public/private status, see
              [Working with Playlists](/documentation/web-api/concepts/playlists)

          uris: A JSON array of the
              [Spotify URIs](/documentation/web-api/concepts/spotify-uris-ids) to add. For
              example:
              `{"uris": ["spotify:track:4iV5W9uYEdYUVa79Axb7Rh","spotify:track:1301WleyT98MSxVHPZCA6M", "spotify:episode:512ojhOuo1ktJprKbVcKyQ"]}`<br/>A
              maximum of 100 items can be added in one request. _**Note**: if the `uris`
              parameter is present in the query string, any URIs listed here in the body will
              be ignored._

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not playlist_id:
            raise ValueError(f"Expected a non-empty value for `playlist_id` but received {playlist_id!r}")
        return self._post(
            f"/playlists/{playlist_id}/tracks",
            body=maybe_transform(
                {
                    "position": position,
                    "published": published,
                    "uris": uris,
                },
                track_add_params.TrackAddParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TrackAddResponse,
        )

    @typing_extensions.deprecated("deprecated")
    def remove(
        self,
        playlist_id: str,
        *,
        tracks: Iterable[track_remove_params.Track],
        published: bool | Omit = omit,
        snapshot_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TrackRemoveResponse:
        """
        **Deprecated:** Use
        [Remove Playlist Items](/documentation/web-api/reference/remove-items-playlist)
        instead.

        Remove one or more items from a user's playlist.

        Args:
          playlist_id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              playlist.

          tracks: An array of objects containing
              [Spotify URIs](/documentation/web-api/concepts/spotify-uris-ids) of the tracks
              or episodes to remove. For example:
              `{ "tracks": [{ "uri": "spotify:track:4iV5W9uYEdYUVa79Axb7Rh" },{ "uri": "spotify:track:1301WleyT98MSxVHPZCA6M" }] }`.
              A maximum of 100 objects can be sent at once.

          published: The playlist's public/private status (if it should be added to the user's
              profile or not): `true` the playlist will be public, `false` the playlist will
              be private, `null` the playlist status is not relevant. For more about
              public/private status, see
              [Working with Playlists](/documentation/web-api/concepts/playlists)

          snapshot_id: The playlist's snapshot ID against which you want to make the changes. The API
              will validate that the specified items exist and in the specified positions and
              make the changes, even if more recent changes have been made to the playlist.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not playlist_id:
            raise ValueError(f"Expected a non-empty value for `playlist_id` but received {playlist_id!r}")
        return self._delete(
            f"/playlists/{playlist_id}/tracks",
            body=maybe_transform(
                {
                    "tracks": tracks,
                    "published": published,
                    "snapshot_id": snapshot_id,
                },
                track_remove_params.TrackRemoveParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TrackRemoveResponse,
        )


class AsyncTracksResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTracksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return AsyncTracksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTracksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return AsyncTracksResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    async def update(
        self,
        playlist_id: str,
        *,
        insert_before: int | Omit = omit,
        published: bool | Omit = omit,
        range_length: int | Omit = omit,
        range_start: int | Omit = omit,
        snapshot_id: str | Omit = omit,
        uris: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TrackUpdateResponse:
        """
        **Deprecated:** Use
        [Update Playlist Items](/documentation/web-api/reference/reorder-or-replace-playlists-items)
        instead.

        Either reorder or replace items in a playlist depending on the request's
        parameters. To reorder items, include `range_start`, `insert_before`,
        `range_length` and `snapshot_id` in the request's body. To replace items,
        include `uris` as either a query parameter or in the request's body. Replacing
        items in a playlist will overwrite its existing items. This operation can be
        used for replacing or clearing items in a playlist. <br/> **Note**: Replace and
        reorder are mutually exclusive operations which share the same endpoint, but
        have different parameters. These operations can't be applied together in a
        single request.

        Args:
          playlist_id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              playlist.

          insert_before: The position where the items should be inserted.<br/>To reorder the items to the
              end of the playlist, simply set _insert_before_ to the position after the last
              item.<br/>Examples:<br/>To reorder the first item to the last position in a
              playlist with 10 items, set _range_start_ to 0, and _insert_before_
              to 10.<br/>To reorder the last item in a playlist with 10 items to the start of
              the playlist, set _range_start_ to 9, and _insert_before_ to 0.

          published: The playlist's public/private status (if it should be added to the user's
              profile or not): `true` the playlist will be public, `false` the playlist will
              be private, `null` the playlist status is not relevant. For more about
              public/private status, see
              [Working with Playlists](/documentation/web-api/concepts/playlists)

          range_length: The amount of items to be reordered. Defaults to 1 if not set.<br/>The range of
              items to be reordered begins from the _range_start_ position, and includes the
              _range_length_ subsequent items.<br/>Example:<br/>To move the items at index
              9-10 to the start of the playlist, _range_start_ is set to 9, and _range_length_
              is set to 2.

          range_start: The position of the first item to be reordered.

          snapshot_id: The playlist's snapshot ID against which you want to make the changes.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not playlist_id:
            raise ValueError(f"Expected a non-empty value for `playlist_id` but received {playlist_id!r}")
        return await self._put(
            f"/playlists/{playlist_id}/tracks",
            body=await async_maybe_transform(
                {
                    "insert_before": insert_before,
                    "published": published,
                    "range_length": range_length,
                    "range_start": range_start,
                    "snapshot_id": snapshot_id,
                    "uris": uris,
                },
                track_update_params.TrackUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TrackUpdateResponse,
        )

    @typing_extensions.deprecated("deprecated")
    def list(
        self,
        playlist_id: str,
        *,
        additional_types: str | Omit = omit,
        fields: str | Omit = omit,
        limit: int | Omit = omit,
        market: str | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[PlaylistTrackObject, AsyncCursorURLPage[PlaylistTrackObject]]:
        """
        **Deprecated:** Use
        [Get Playlist Items](/documentation/web-api/reference/get-playlists-items)
        instead.

        Get full details of the items of a playlist owned by a Spotify user.

        Args:
          playlist_id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              playlist.

          additional_types: A comma-separated list of item types that your client supports besides the
              default `track` type. Valid types are: `track` and `episode`.<br/> _**Note**:
              This parameter was introduced to allow existing clients to maintain their
              current behaviour and might be deprecated in the future._<br/> In addition to
              providing this parameter, make sure that your client properly handles cases of
              new types in the future by checking against the `type` field of each object.

          fields: Filters for the query: a comma-separated list of the fields to return. If
              omitted, all fields are returned. For example, to get just the total number of
              items and the request limit:<br/>`fields=total,limit`<br/>A dot separator can be
              used to specify non-reoccurring fields, while parentheses can be used to specify
              reoccurring fields within objects. For example, to get just the added date and
              user ID of the adder:<br/>`fields=items(added_at,added_by.id)`<br/>Use multiple
              parentheses to drill down into nested objects, for
              example:<br/>`fields=items(track(name,href,album(name,href)))`<br/>Fields can be
              excluded by prefixing them with an exclamation mark, for
              example:<br/>`fields=items.track.album(!external_urls,images)`

          limit: The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 100.

          market: An
              [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
              If a country code is specified, only content that is available in that market
              will be returned.<br/> If a valid user access token is specified in the request
              header, the country associated with the user account will take priority over
              this parameter.<br/> _**Note**: If neither market or user country are provided,
              the content is considered unavailable for the client._<br/> Users can view the
              country that is associated with their account in the
              [account settings](https://www.spotify.com/account/overview/).

          offset: The index of the first item to return. Default: 0 (the first item). Use with
              limit to get the next set of items.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not playlist_id:
            raise ValueError(f"Expected a non-empty value for `playlist_id` but received {playlist_id!r}")
        return self._get_api_list(
            f"/playlists/{playlist_id}/tracks",
            page=AsyncCursorURLPage[PlaylistTrackObject],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "additional_types": additional_types,
                        "fields": fields,
                        "limit": limit,
                        "market": market,
                        "offset": offset,
                    },
                    track_list_params.TrackListParams,
                ),
            ),
            model=PlaylistTrackObject,
        )

    @typing_extensions.deprecated("deprecated")
    async def add(
        self,
        playlist_id: str,
        *,
        position: int | Omit = omit,
        published: bool | Omit = omit,
        uris: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TrackAddResponse:
        """
        **Deprecated:** Use
        [Add Items to Playlist](/documentation/web-api/reference/add-items-to-playlist)
        instead.

        Add one or more items to a user's playlist.

        Args:
          playlist_id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              playlist.

          position: The position to insert the items, a zero-based index. For example, to insert the
              items in the first position: `position=0` ; to insert the items in the third
              position: `position=2`. If omitted, the items will be appended to the playlist.
              Items are added in the order they appear in the uris array. For example:
              `{"uris": ["spotify:track:4iV5W9uYEdYUVa79Axb7Rh","spotify:track:1301WleyT98MSxVHPZCA6M"], "position": 3}`

          published: The playlist's public/private status (if it should be added to the user's
              profile or not): `true` the playlist will be public, `false` the playlist will
              be private, `null` the playlist status is not relevant. For more about
              public/private status, see
              [Working with Playlists](/documentation/web-api/concepts/playlists)

          uris: A JSON array of the
              [Spotify URIs](/documentation/web-api/concepts/spotify-uris-ids) to add. For
              example:
              `{"uris": ["spotify:track:4iV5W9uYEdYUVa79Axb7Rh","spotify:track:1301WleyT98MSxVHPZCA6M", "spotify:episode:512ojhOuo1ktJprKbVcKyQ"]}`<br/>A
              maximum of 100 items can be added in one request. _**Note**: if the `uris`
              parameter is present in the query string, any URIs listed here in the body will
              be ignored._

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not playlist_id:
            raise ValueError(f"Expected a non-empty value for `playlist_id` but received {playlist_id!r}")
        return await self._post(
            f"/playlists/{playlist_id}/tracks",
            body=await async_maybe_transform(
                {
                    "position": position,
                    "published": published,
                    "uris": uris,
                },
                track_add_params.TrackAddParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TrackAddResponse,
        )

    @typing_extensions.deprecated("deprecated")
    async def remove(
        self,
        playlist_id: str,
        *,
        tracks: Iterable[track_remove_params.Track],
        published: bool | Omit = omit,
        snapshot_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TrackRemoveResponse:
        """
        **Deprecated:** Use
        [Remove Playlist Items](/documentation/web-api/reference/remove-items-playlist)
        instead.

        Remove one or more items from a user's playlist.

        Args:
          playlist_id: The [Spotify ID](/documentation/web-api/concepts/spotify-uris-ids) of the
              playlist.

          tracks: An array of objects containing
              [Spotify URIs](/documentation/web-api/concepts/spotify-uris-ids) of the tracks
              or episodes to remove. For example:
              `{ "tracks": [{ "uri": "spotify:track:4iV5W9uYEdYUVa79Axb7Rh" },{ "uri": "spotify:track:1301WleyT98MSxVHPZCA6M" }] }`.
              A maximum of 100 objects can be sent at once.

          published: The playlist's public/private status (if it should be added to the user's
              profile or not): `true` the playlist will be public, `false` the playlist will
              be private, `null` the playlist status is not relevant. For more about
              public/private status, see
              [Working with Playlists](/documentation/web-api/concepts/playlists)

          snapshot_id: The playlist's snapshot ID against which you want to make the changes. The API
              will validate that the specified items exist and in the specified positions and
              make the changes, even if more recent changes have been made to the playlist.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not playlist_id:
            raise ValueError(f"Expected a non-empty value for `playlist_id` but received {playlist_id!r}")
        return await self._delete(
            f"/playlists/{playlist_id}/tracks",
            body=await async_maybe_transform(
                {
                    "tracks": tracks,
                    "published": published,
                    "snapshot_id": snapshot_id,
                },
                track_remove_params.TrackRemoveParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TrackRemoveResponse,
        )


class TracksResourceWithRawResponse:
    def __init__(self, tracks: TracksResource) -> None:
        self._tracks = tracks

        self.update = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                tracks.update,  # pyright: ignore[reportDeprecated],
            )
        )
        self.list = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                tracks.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.add = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                tracks.add,  # pyright: ignore[reportDeprecated],
            )
        )
        self.remove = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                tracks.remove,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncTracksResourceWithRawResponse:
    def __init__(self, tracks: AsyncTracksResource) -> None:
        self._tracks = tracks

        self.update = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                tracks.update,  # pyright: ignore[reportDeprecated],
            )
        )
        self.list = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                tracks.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.add = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                tracks.add,  # pyright: ignore[reportDeprecated],
            )
        )
        self.remove = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                tracks.remove,  # pyright: ignore[reportDeprecated],
            )
        )


class TracksResourceWithStreamingResponse:
    def __init__(self, tracks: TracksResource) -> None:
        self._tracks = tracks

        self.update = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                tracks.update,  # pyright: ignore[reportDeprecated],
            )
        )
        self.list = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                tracks.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.add = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                tracks.add,  # pyright: ignore[reportDeprecated],
            )
        )
        self.remove = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                tracks.remove,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncTracksResourceWithStreamingResponse:
    def __init__(self, tracks: AsyncTracksResource) -> None:
        self._tracks = tracks

        self.update = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                tracks.update,  # pyright: ignore[reportDeprecated],
            )
        )
        self.list = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                tracks.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.add = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                tracks.add,  # pyright: ignore[reportDeprecated],
            )
        )
        self.remove = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                tracks.remove,  # pyright: ignore[reportDeprecated],
            )
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .shared.show_base import ShowBase
from .shared.simplified_episode_object import SimplifiedEpisodeObject

__all__ = ["ShowRetrieveResponse", "ShowRetrieveResponseEpisodes"]


class ShowRetrieveResponseEpisodes(BaseModel):
    """The episodes of the show."""

    href: str
    """A link to the Web API endpoint returning the full result of the request"""

    limit: int
    """
    The maximum number of items in the response (as set in the query or by default).
    """

    next: Optional[str] = None
    """URL to the next page of items. ( `null` if none)"""

    offset: int
    """The offset of the items returned (as set in the query or by default)"""

    previous: Optional[str] = None
    """URL to the previous page of items. ( `null` if none)"""

    total: int
    """The total number of items available to return."""

    items: Optional[List[SimplifiedEpisodeObject]] = None

    published: Optional[bool] = None
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """


class ShowRetrieveResponse(ShowBase):
    episodes: ShowRetrieveResponseEpisodes
    """The episodes of the show."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from spotted import Spotted, AsyncSpotted
from tests.utils import assert_matches_type
from spotted.types.playlists import FollowerCheckResponse

# pyright: reportDeprecated=false

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestFollowers:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_check(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            follower = client.playlists.followers.check(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            )

        assert_matches_type(FollowerCheckResponse, follower, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_check_with_all_params(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            follower = client.playlists.followers.check(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
                ids="jmperezperez",
            )

        assert_matches_type(FollowerCheckResponse, follower, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_check(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.playlists.followers.with_raw_response.check(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        follower = response.parse()
        assert_matches_type(FollowerCheckResponse, follower, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_check(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            with client.playlists.followers.with_streaming_response.check(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                follower = response.parse()
                assert_matches_type(FollowerCheckResponse, follower, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_check(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `playlist_id` but received ''"):
                client.playlists.followers.with_raw_response.check(
                    playlist_id="",
                )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_follow(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            follower = client.playlists.followers.follow(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            )

        assert follower is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_follow_with_all_params(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            follower = client.playlists.followers.follow(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
                published=True,
            )

        assert follower is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_follow(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.playlists.followers.with_raw_response.follow(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        follower = response.parse()
        assert follower is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_follow(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            with client.playlists.followers.with_streaming_response.follow(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                follower = response.parse()
                assert follower is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_follow(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `playlist_id` but received ''"):
                client.playlists.followers.with_raw_response.follow(
                    playlist_id="",
                )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_unfollow(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            follower = client.playlists.followers.unfollow(
                "3cEYpjA9oz9GiPac4AsH4n",
            )

        assert follower is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_unfollow(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.playlists.followers.with_raw_response.unfollow(
                "3cEYpjA9oz9GiPac4AsH4n",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        follower = response.parse()
        assert follower is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_unfollow(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            with client.playlists.followers.with_streaming_response.unfollow(
                "3cEYpjA9oz9GiPac4AsH4n",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                follower = response.parse()
                assert follower is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_unfollow(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `playlist_id` but received ''"):
                client.playlists.followers.with_raw_response.unfollow(
                    "",
                )


class TestAsyncFollowers:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_check(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            follower = await async_client.playlists.followers.check(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            )

        assert_matches_type(FollowerCheckResponse, follower, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_check_with_all_params(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            follower = await async_client.playlists.followers.check(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
                ids="jmperezperez",
            )

        assert_matches_type(FollowerCheckResponse, follower, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_check(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.playlists.followers.with_raw_response.check(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        follower = await response.parse()
        assert_matches_type(FollowerCheckResponse, follower, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_check(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.playlists.followers.with_streaming_response.check(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                follower = await response.parse()
                assert_matches_type(FollowerCheckResponse, follower, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_check(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `playlist_id` but received ''"):
                await async_client.playlists.followers.with_raw_response.check(
                    playlist_id="",
                )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_follow(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            follower = await async_client.playlists.followers.follow(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            )

        assert follower is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_follow_with_all_params(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            follower = await async_client.playlists.followers.follow(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
                published=True,
            )

        assert follower is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_follow(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.playlists.followers.with_raw_response.follow(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        follower = await response.parse()
        assert follower is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_follow(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.playlists.followers.with_streaming_response.follow(
                playlist_id="3cEYpjA9oz9GiPac4AsH4n",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                follower = await response.parse()
                assert follower is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_follow(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `playlist_id` but received ''"):
                await async_client.playlists.followers.with_raw_response.follow(
                    playlist_id="",
                )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_unfollow(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            follower = await async_client.playlists.followers.unfollow(
                "3cEYpjA9oz9GiPac4AsH4n",
            )

        assert follower is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_unfollow(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.playlists.followers.with_raw_response.unfollow(
                "3cEYpjA9oz9GiPac4AsH4n",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        follower = await response.parse()
        assert follower is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_unfollow(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.playlists.followers.with_streaming_response.unfollow(
                "3cEYpjA9oz9GiPac4AsH4n",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                follower = await response.parse()
                assert follower is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_unfollow(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `playlist_id` but received ''"):
                await async_client.playlists.followers.with_raw_response.unfollow(
                    "",
                )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from spotted import Spotted, AsyncSpotted
from tests.utils import assert_matches_type
from spotted.types import SearchQueryResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSearch:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_query(self, client: Spotted) -> None:
        search = client.search.query(
            q="remaster%20track:Doxy%20artist:Miles%20Davis",
            type=["album"],
        )
        assert_matches_type(SearchQueryResponse, search, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_query_with_all_params(self, client: Spotted) -> None:
        search = client.search.query(
            q="remaster%20track:Doxy%20artist:Miles%20Davis",
            type=["album"],
            include_external="audio",
            limit=10,
            market="ES",
            offset=5,
        )
        assert_matches_type(SearchQueryResponse, search, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_query(self, client: Spotted) -> None:
        response = client.search.with_raw_response.query(
            q="remaster%20track:Doxy%20artist:Miles%20Davis",
            type=["album"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        search = response.parse()
        assert_matches_type(SearchQueryResponse, search, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_query(self, client: Spotted) -> None:
        with client.search.with_streaming_response.query(
            q="remaster%20track:Doxy%20artist:Miles%20Davis",
            type=["album"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            search = response.parse()
            assert_matches_type(SearchQueryResponse, search, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncSearch:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_query(self, async_client: AsyncSpotted) -> None:
        search = await async_client.search.query(
            q="remaster%20track:Doxy%20artist:Miles%20Davis",
            type=["album"],
        )
        assert_matches_type(SearchQueryResponse, search, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_query_with_all_params(self, async_client: AsyncSpotted) -> None:
        search = await async_client.search.query(
            q="remaster%20track:Doxy%20artist:Miles%20Davis",
            type=["album"],
            include_external="audio",
            limit=10,
            market="ES",
            offset=5,
        )
        assert_matches_type(SearchQueryResponse, search, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_query(self, async_client: AsyncSpotted) -> None:
        response = await async_client.search.with_raw_response.query(
            q="remaster%20track:Doxy%20artist:Miles%20Davis",
            type=["album"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        search = await response.parse()
        assert_matches_type(SearchQueryResponse, search, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_query(self, async_client: AsyncSpotted) -> None:
        async with async_client.search.with_streaming_response.query(
            q="remaster%20track:Doxy%20artist:Miles%20Davis",
            type=["album"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            search = await response.parse()
            assert_matches_type(SearchQueryResponse, search, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from spotted import Spotted, AsyncSpotted
from tests.utils import assert_matches_type
from spotted.types.me import (
    ShowListResponse,
    ShowCheckResponse,
)
from spotted.pagination import SyncCursorURLPage, AsyncCursorURLPage

# pyright: reportDeprecated=false

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestShows:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Spotted) -> None:
        show = client.me.shows.list()
        assert_matches_type(SyncCursorURLPage[ShowListResponse], show, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Spotted) -> None:
        show = client.me.shows.list(
            limit=10,
            offset=5,
        )
        assert_matches_type(SyncCursorURLPage[ShowListResponse], show, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Spotted) -> None:
        response = client.me.shows.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        show = response.parse()
        assert_matches_type(SyncCursorURLPage[ShowListResponse], show, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Spotted) -> None:
        with client.me.shows.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            show = response.parse()
            assert_matches_type(SyncCursorURLPage[ShowListResponse], show, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_check(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            show = client.me.shows.check(
                ids="5CfCWKI5pZ28U0uOzXkDHe,5as3aKmN2k11yfDDDSrvaZ",
            )

        assert_matches_type(ShowCheckResponse, show, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_check(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.me.shows.with_raw_response.check(
                ids="5CfCWKI5pZ28U0uOzXkDHe,5as3aKmN2k11yfDDDSrvaZ",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        show = response.parse()
        assert_matches_type(ShowCheckResponse, show, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_check(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            with client.me.shows.with_streaming_response.check(
                ids="5CfCWKI5pZ28U0uOzXkDHe,5as3aKmN2k11yfDDDSrvaZ",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                show = response.parse()
                assert_matches_type(ShowCheckResponse, show, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_remove(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            show = client.me.shows.remove()

        assert show is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_remove_with_all_params(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            show = client.me.shows.remove(
                ids=["string"],
                published=True,
            )

        assert show is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_remove(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.me.shows.with_raw_response.remove()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        show = response.parse()
        assert show is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_remove(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            with client.me.shows.with_streaming_response.remove() as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                show = response.parse()
                assert show is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_save(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            show = client.me.shows.save()

        assert show is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_save_with_all_params(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            show = client.me.shows.save(
                ids=["string"],
                published=True,
            )

        assert show is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_save(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.me.shows.with_raw_response.save()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        show = response.parse()
        assert show is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_save(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            with client.me.shows.with_streaming_response.save() as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                show = response.parse()
                assert show is None

        assert cast(Any, response.is_closed) is True


class TestAsyncShows:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncSpotted) -> None:
        show = await async_client.me.shows.list()
        assert_matches_type(AsyncCursorURLPage[ShowListResponse], show, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncSpotted) -> None:
        show = await async_client.me.shows.list(
            limit=10,
            offset=5,
        )
        assert_matches_type(AsyncCursorURLPage[ShowListResponse], show, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSpotted) -> None:
        response = await async_client.me.shows.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        show = await response.parse()
        assert_matches_type(AsyncCursorURLPage[ShowListResponse], show, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSpotted) -> None:
        async with async_client.me.shows.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            show = await response.parse()
            assert_matches_type(AsyncCursorURLPage[ShowListResponse], show, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_check(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            show = await async_client.me.shows.check(
                ids="5CfCWKI5pZ28U0uOzXkDHe,5as3aKmN2k11yfDDDSrvaZ",
            )

        assert_matches_type(ShowCheckResponse, show, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_check(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.me.shows.with_raw_response.check(
                ids="5CfCWKI5pZ28U0uOzXkDHe,5as3aKmN2k11yfDDDSrvaZ",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        show = await response.parse()
        assert_matches_type(ShowCheckResponse, show, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_check(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.me.shows.with_streaming_response.check(
                ids="5CfCWKI5pZ28U0uOzXkDHe,5as3aKmN2k11yfDDDSrvaZ",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                show = await response.parse()
                assert_matches_type(ShowCheckResponse, show, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_remove(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            show = await async_client.me.shows.remove()

        assert show is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_remove_with_all_params(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            show = await async_client.me.shows.remove(
                ids=["string"],
                published=True,
            )

        assert show is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_remove(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.me.shows.with_raw_response.remove()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        show = await response.parse()
        assert show is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_remove(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.me.shows.with_streaming_response.remove() as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                show = await response.parse()
                assert show is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_save(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            show = await async_client.me.shows.save()

        assert show is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_save_with_all_params(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            show = await async_client.me.shows.save(
                ids=["string"],
                published=True,
            )

        assert show is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_save(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.me.shows.with_raw_response.save()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        show = await response.parse()
        assert show is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_save(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.me.shows.with_streaming_response.save() as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                show = await response.parse()
                assert show is None

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .users import (
    UsersResource,
    AsyncUsersResource,
    UsersResourceWithRawResponse,
    AsyncUsersResourceWithRawResponse,
    UsersResourceWithStreamingResponse,
    AsyncUsersResourceWithStreamingResponse,
)
from .playlists import (
    PlaylistsResource,
    AsyncPlaylistsResource,
    PlaylistsResourceWithRawResponse,
    AsyncPlaylistsResourceWithRawResponse,
    PlaylistsResourceWithStreamingResponse,
    AsyncPlaylistsResourceWithStreamingResponse,
)

__all__ = [
    "PlaylistsResource",
    "AsyncPlaylistsResource",
    "PlaylistsResourceWithRawResponse",
    "AsyncPlaylistsResourceWithRawResponse",
    "PlaylistsResourceWithStreamingResponse",
    "AsyncPlaylistsResourceWithStreamingResponse",
    "UsersResource",
    "AsyncUsersResource",
    "UsersResourceWithRawResponse",
    "AsyncUsersResourceWithRawResponse",
    "UsersResourceWithStreamingResponse",
    "AsyncUsersResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["FollowingBulkRetrieveParams"]


class FollowingBulkRetrieveParams(TypedDict, total=False):
    type: Required[Literal["artist"]]
    """The ID type: currently only `artist` is supported."""

    after: str
    """The last artist ID retrieved from the previous request."""

    limit: int
    """The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["EpisodeListParams"]


class EpisodeListParams(TypedDict, total=False):
    limit: int
    """The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50."""

    market: str
    """
    An
    [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
    If a country code is specified, only content that is available in that market
    will be returned.<br/> If a valid user access token is specified in the request
    header, the country associated with the user account will take priority over
    this parameter.<br/> _**Note**: If neither market or user country are provided,
    the content is considered unavailable for the client._<br/> Users can view the
    country that is associated with their account in the
    [account settings](https://www.spotify.com/account/overview/).
    """

    offset: int
    """The index of the first item to return.

    Default: 0 (the first item). Use with limit to get the next set of items.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["AudiobookRemoveParams"]


class AudiobookRemoveParams(TypedDict, total=False):
    ids: Required[str]
    """
    A comma-separated list of the
    [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids). For example:
    `ids=18yVqkdbdRvS24c0Ilj2ci,1HGw3J3NxZO1TP1BTtVhpZ`. Maximum: 50 IDs.
    """
